package ca.odell.glazedlists.impl.gui;

import ca.odell.glazedlists.impl.gui.SortingState;
import java.util.List;

/**
 * @see ca.odell.glazedlists.gui.AbstractTableComparatorChooser.MULTIPLE_COLUMN_KEYBOARD
 * 
 * @author <a href="mailto:jesse@swank.ca">Jesse Wilson</a>
 */
public class MouseKeyboardSortingStrategy implements ca.odell.glazedlists.impl.gui.SortingStrategy {
    
    public MouseKeyboardSortingStrategy() {
        super();
    }
    
    /**
     * a column is sorted in forward, reverse or not at all 
     */
    private static final int NONE = 0;
    private static final int FORWARD = 1;
    private static final int REVERSE = 2;
    
    /**
     * Adjust the sorting state based on receiving the specified click event.
     */
    public void columnClicked(SortingState sortingState, int column, int clicks, boolean shift, boolean control) {
        SortingState.SortingColumn sortingColumn = (SortingState.SortingColumn)sortingState.getColumns().get(column);
        if (sortingColumn.getComparators().isEmpty()) return;
        List recentlyClickedColumns = sortingState.getRecentlyClickedColumns();
        int comparatorIndexBefore = sortingColumn.getComparatorIndex();
        final int forwardReverseNoneBefore;
        if (comparatorIndexBefore == -1) forwardReverseNoneBefore = NONE; else forwardReverseNoneBefore = sortingColumn.isReverse() ? REVERSE : FORWARD;
        int forwardReverseNoneAfter;
        int comparatorIndexAfter;
        boolean moreComparators = comparatorIndexBefore + 1 < sortingColumn.getComparators().size();
        boolean lastDirective = shift ? forwardReverseNoneBefore == FORWARD : forwardReverseNoneBefore == REVERSE;
        if (moreComparators && lastDirective) {
            comparatorIndexAfter = (comparatorIndexBefore + 1) % sortingColumn.getComparators().size();
            forwardReverseNoneAfter = forwardReverseNoneBefore == FORWARD ? REVERSE : FORWARD;
        } else {
            comparatorIndexAfter = comparatorIndexBefore != -1 ? comparatorIndexBefore : 0;
            forwardReverseNoneAfter = (shift ? forwardReverseNoneBefore + 2 : forwardReverseNoneBefore + 1) % 3;
        }
        if (!control) {
            sortingState.clearComparators();
        }
        if (forwardReverseNoneAfter == NONE) {
            sortingColumn.clear();
            recentlyClickedColumns.remove(sortingColumn);
        } else {
            sortingColumn.setComparatorIndex(comparatorIndexAfter);
            sortingColumn.setReverse(forwardReverseNoneAfter == REVERSE);
            if (!recentlyClickedColumns.contains(sortingColumn)) {
                recentlyClickedColumns.add(sortingColumn);
            }
        }
        sortingState.fireSortingChanged();
    }
}
