package ca.odell.glazedlists;

import ca.odell.glazedlists.event.ListEvent;

/**
 * A list that allows nested events to be managed externally. This is very useful
 * in testing handling of sophisticated events, because it allows big events
 * to be composed from simpler ones.
 */
public class ExternalNestingEventList extends TransformedList {
    final boolean forward;
    
    public ExternalNestingEventList(EventList source) {
        this(source, false);
    }
    
    public ExternalNestingEventList(EventList source, boolean forward) {
        super(source);
        this.forward = forward;
        source.addListEventListener(this);
    }
    
    public void beginEvent(boolean allowNested) {
        updates.beginEvent(allowNested);
    }
    
    public void commitEvent() {
        updates.commitEvent();
    }
    
    protected boolean isWritable() {
        return true;
    }
    
    public void listChanged(ListEvent listChanges) {
        if (forward) {
            updates.forwardEvent(listChanges);
        } else {
            if (listChanges.isReordering()) {
                int[] reorderMap = listChanges.getReorderMap();
                updates.reorder(reorderMap);
            } else {
                while (listChanges.next()) {
                    updates.addChange(listChanges.getType(), listChanges.getIndex());
                }
            }
        }
    }
}
