package ca.odell.glazedlists;

import ca.odell.glazedlists.matchers.Matcher;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Comparator;
import java.io.*;

/**
 * A factory class useful for testing!
 *
 * @author <a href="mailto:jesse@swank.ca">Jesse Wilson</a>
 */
public class GlazedListsTests {
    
    /**
     * A dummy constructor to prevent instantiation of this class
     */
    private GlazedListsTests() {
        super();
        throw new UnsupportedOperationException();
    }
    
    /**
     * Convert the characters of the specified String to a list.
     */
    public static List stringToList(CharSequence chars) {
        List result = new ArrayList(chars.length());
        for (int i = 0; i < chars.length(); i++) {
            result.add(chars.subSequence(i, i + 1).toString());
        }
        return result;
    }
    
    /**
     * Convert a String like "AA,BB,CDE" into three Lists:
     * { [ A A ], [ B B ], [ C D E ] }
     */
    public static List stringToLists(CharSequence chars) {
        List result = new ArrayList();
        String[] strings = chars.toString().split(",");
        for (int i = 0; i < strings.length; i++) {
            result.add(stringToList(strings[i]));
        }
        return result;
    }
    
    /**
     * Convert an array of Strings into a List of characters.
     */
    public static List stringsToList(CharSequence[] data) {
        List result = new ArrayList();
        for (int s = 0; s < data.length; s++) {
            result.addAll(stringToList(data[s]));
        }
        return result;
    }
    
    /**
     * Convert the specified int[] array to a List of Integers.
     */
    public static List intArrayToIntegerCollection(int[] values) {
        List result = new ArrayList();
        for (int i = 0; i < values.length; i++) {
            result.add(new Integer(values[i]));
        }
        return result;
    }
    
    /**
     * Manually apply the specified filter to the specified list.
     */
    public static List filter(List input, Matcher matcher) {
        List result = new ArrayList();
        for (Iterator i = input.iterator(); i.hasNext(); ) {
            Object element = i.next();
            if (matcher.matches(element)) result.add(element);
        }
        return result;
    }
    
    /**
     * This matcher matches everything greater than its minimum.
     */
    public static Matcher matchAtLeast(int minimum) {
        return new AtLeastMatcher(minimum);
    }
    
    /**
     * Serialize the specified object to bytes, then deserialize it back.
     */
    public static Object serialize(Object object) throws IOException, ClassNotFoundException {
        return (Object)fromBytes(toBytes(object));
    }
    
    public static byte[] toBytes(Object object) throws IOException {
        ByteArrayOutputStream bytesOut = new ByteArrayOutputStream();
        ObjectOutputStream objectsOut = new ObjectOutputStream(bytesOut);
        objectsOut.writeObject(object);
        return bytesOut.toByteArray();
    }
    
    public static Object fromBytes(byte[] bytes) throws IOException, ClassNotFoundException {
        ByteArrayInputStream bytesIn = new ByteArrayInputStream(bytes);
        ObjectInputStream objectsIn = new ObjectInputStream(bytesIn);
        return objectsIn.readObject();
    }
    
    public static String toString(byte[] bytes) {
        StringBuffer result = new StringBuffer();
        for (int b = 0; b < bytes.length; b++) {
            result.append(bytes[b] < 0 ? "-" : " ");
            String hexString = Integer.toString(Math.abs(bytes[b]), 16);
            while (hexString.length() < 2) hexString = "0" + hexString;
            result.append("0x").append(hexString).append(", ");
            if (b % 16 == 15) result.append("\n");
        }
        return result.toString();
    }
    
    private static class AtLeastMatcher implements Matcher {
        private final int minimum;
        
        public AtLeastMatcher(int minimum) {
            super();
            this.minimum = minimum;
        }
        
        public boolean matches(Object x0) {
            Integer value = (Integer)x0;
            return value.intValue() >= minimum;
        }
        /*missing*/
    }
    
    /**
     * A comparator for comparing integer arrays, which are particularly well
     * suited to sorting and filtering tests.
     */
    public static Comparator intArrayComparator(int index) {
        return new IntArrayComparator(index);
    }
    
    private static class IntArrayComparator implements Comparator {
        public int index;
        
        public IntArrayComparator(int index) {
            super();
            this.index = index;
        }
        
        public int compare(Object x0, Object x1) {
            int[] a = (int[])x0;
            int[] b = (int[])x1;
            return a[index] - b[index];
        }
        /*missing*/
    }
}
